#include <iostream>

#include <Eigen/Dense>

Eigen::MatrixXd GetA(const Eigen::VectorXd &x) {
	int n = x.size();
	Eigen::MatrixXd A(n, 3);
	
	for (int i = 0; i < n; ++i) {
		A(i, 0) = 1.0;
		A(i, 1) = x(i);
		A(i, 2) = x(i) * x(i);
	}
	
	return A;
}

Eigen::VectorXd LeastSquares(const Eigen::MatrixXd &A, const Eigen::VectorXd &b) {
	return (A.transpose() * A).ldlt().solve(A.transpose() * b);
}

Eigen::MatrixXd BestApprox(const Eigen::MatrixXd &B, int k) {
	Eigen::JacobiSVD<Eigen::MatrixXd> svd(B, Eigen::ComputeThinU | Eigen::ComputeThinV);

	Eigen::VectorXd s = svd.singularValues();
	s.tail(s.size() - k) = Eigen::VectorXd::Zero(s.size() - k);
	Eigen::MatrixXd S = s.asDiagonal();

	return svd.matrixU() * S * svd.matrixV().transpose();
}

double FitLineThroughOrigin(const Eigen::MatrixXd &B) {
	Eigen::MatrixXd BB = BestApprox(B, 1);
	double a = BB.row(1).sum() / BB.row(0).sum();
	return a;
}

int main() {
	// Test of Task (b):
	Eigen::VectorXd x(10), y(10);
	x << -1.0, -0.8, -0.6, -0.3, -0.1, 0.1,  0.3, 0.6, 0.8, 1.0;
	y <<  0.0, -0.5, -1.0, -1.1, -1.1,-1.0, -0.5, 0.2, 1.1, 1.9;
	Eigen::MatrixXd A = GetA(x);
	Eigen::VectorXd b = y;
	
	// Test of Task (c):
	Eigen::VectorXd c = LeastSquares(A, b);
	std::cout << "p_c(x) = " << c(2) << " x^2 + " << c(1) << " x + " << c(0) << std::endl;

	// Test of Task (d):
	int k = 2;
	Eigen::MatrixXd B = Eigen::MatrixXd::Identity(4, 3);
	Eigen::MatrixXd Bk = BestApprox(B, k);
	std::cout << "The best rank-" << k << " approximation of" << std::endl
			<< B << std::endl << "is" << std::endl << Bk <<std::endl;

	// Test of Task (e):
	Eigen::MatrixXd B_(2, 6);
	B_ << 0.2, 0.4 , 0.8, 1.0 , 1.2, 1.4,
	      0.2, 0.25, 0.3, 0.37, 0.5, 0.8;
	std::cout << "The slope of the fitted line through the origin is" << std::endl;
	std::cout << "a = " << FitLineThroughOrigin(B_) << std::endl;

	return 0;
}
