#include <cmath>
#include <iostream>
#include <functional>
#include <vector>

double GaussChebyshev(const std::function<double(double)> &f, int n) {
    double q = 0.0;
    double A = M_PI/n;

	std::vector<double> x(n);
	for (int i = 0; i < n; ++i) {
        x[i] = std::cos( (i + 0.5) * M_PI / n);
		q += A * f(x[i]);
	}

	return q;
}

double f1(double x) {
    return std::pow(x,8.0);
}
double f2(double x) {
    return std::pow(x,10.0);
}

int main() {
    double I_1 = 35*M_PI/128; //Exact solution for f(x) = x^8
    double I_2 = 63*M_PI/256; //Exact solution for f(x) = x^10

    double u1, u2, error_1, error_2;
    u1 = GaussChebyshev(f1, 5);
    error_1=std::abs(u1-I_1); //Error in approximation for f(x) = x^8
    std::cout << "For f(x)=x^8, Q_n(f): " << u1 << " ;   Error: " << error_1 << std::endl;
    u2 = GaussChebyshev(f2, 5);
    error_2=std::abs(u2-I_2); //Error in approximation for f(x) = x^10
    std::cout << "For f(x)=x^10, Q_n(f):  " << u2 << " ;   Error: " << error_2 << std::endl;

	return 0;
}

